package routes

import (
	"fmt"
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
)

type Oauth2 struct {
	controller.Controller
}

// Authorize is protected by auth required middleware. It performs Oauth
// authorization for logged-in users.
func (c *Oauth2) Authorize(ctx echo.Context) error {
	user, ok := ctx.Get(context.AuthenticatedUserKey).(sqlc.User)
	if !ok {
		return c.BadReq("Missing authenticated user")
	}

	r := ctx.Request()
	updatedContext := context.WithValue(r.Context(), "oauth-user-id", strconv.Itoa(user.ID))
	updatedRequest := r.WithContext(updatedContext)
	ctx.SetRequest(updatedRequest)

	err := c.Controller.Container.OAuth2.OAuth2Server.
		HandleAuthorizeRequest(ctx.Response(), ctx.Request())
	if err != nil {
		echoErr := echo.NewHTTPError(http.StatusBadRequest, "Bad Request")
		echoErr.SetInternal(err)
		return echoErr
	}

	return nil
}

func (c *Oauth2) Token(ctx echo.Context) error {
	err := c.Controller.Container.OAuth2.OAuth2Server.
		HandleTokenRequest(ctx.Response(), ctx.Request())
	if err != nil {
		return c.BadReq(err.Error())
	}

	return nil
}

// Lookup returns additional information used by Discourse when performing an
// OAuth2 login. Right now this is only the users email address.
func (c *Oauth2) Lookup(ctx echo.Context) error {
	fmt.Println("LOOKUP CALLED")
	fmt.Printf("%+v\n", ctx.Request().Header)

	userID := ctx.Param("user")
	token := ctx.Param("token")

	userIDParsed, err := strconv.Atoi(userID)
	if err != nil {
		return c.BadReq("Invalid userID")
	}

	ti, err := c.Controller.Container.OAuth2.OAuth2Server.Manager.LoadAccessToken(ctx.Request().Context(), token)
	if err != nil {
		return c.BadReq(err.Error())
	}

	if ti.GetUserID() != userID {
		return c.BadReq("UserID mismatch")
	}

	user, err := c.Container.DB.C.GetUserByID(ctx.Request().Context(), userIDParsed)
	if err != nil {
		return c.BadReq("Invalid userID")
	}

	return ctx.JSON(http.StatusOK, map[string]string{
		"user_id": userID,
		"name":    user.Name.String,
		"email":   user.Email,
	})
}
