package services

import (
	"strings"

	"github.com/microcosm-cc/bluemonday"
)

type Sanitizer struct {
	p         bluemonday.Policy
	basicHtml bluemonday.Policy
	rss       bluemonday.Policy
}

func NewSanitizer() *Sanitizer {
	htmlSanitizer := bluemonday.UGCPolicy()
	htmlSanitizer.AddTargetBlankToFullyQualifiedLinks(true)

	return &Sanitizer{
		p:         *bluemonday.StrictPolicy(),
		basicHtml: *htmlSanitizer,
	}
}

func (s *Sanitizer) Sanitize(i string) string {
	return s.p.Sanitize(i)
}

func (s *Sanitizer) SanitizeHTML(i string) string {
	return s.basicHtml.Sanitize(i)
}

func (s *Sanitizer) SanitizeRSS(i string) string {
	return strings.ReplaceAll(i, "<br>", "")
}
